using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// VAMLock v1.0
//
// Locks different parameters for a "Person" atom
// The original code snippet comes from VRStudy ( https://hub.virtamate.com/members/vrstudy.2961/ )
// Thank you very much for that !

namespace VAMLockPlugin
{
    public class VAMLock : MVRScript
    {
		
		private JSONStorableBool autolockAppearance;
		private JSONStorableBool autolockPose;
		private JSONStorableBool autolockFemaleGlutePhysics;
		private JSONStorableBool autolockFemaleBreastPhysics;
		private JSONStorableBool autolockPlugin;
		private JSONStorableBool autolockSkin;
		private JSONStorableBool autolockMorph;
		private JSONStorableBool autolockHair;
		private JSONStorableBool autolockClothing;
				
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "Person")
                {
                    SuperController.LogError("Please add VAMLock on an Person Atom");
                    return;
                }
				
				autolockAppearance = new JSONStorableBool("Autolock Appearance", false);
				autolockAppearance.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockAppearance, false);
				RegisterBool(autolockAppearance);
				
				autolockPose = new JSONStorableBool("Autolock Pose", false);
				autolockPose.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockPose, false);
				RegisterBool(autolockPose);
				
				autolockFemaleGlutePhysics = new JSONStorableBool("Autolock Female Glute Physics", false);
				autolockFemaleGlutePhysics.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockFemaleGlutePhysics, false);
				RegisterBool(autolockFemaleGlutePhysics);
				
				autolockFemaleBreastPhysics = new JSONStorableBool("Autolock Female Breast Physics", false);
				autolockFemaleBreastPhysics.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockFemaleBreastPhysics, false);
				RegisterBool(autolockFemaleBreastPhysics);
				
				autolockPlugin = new JSONStorableBool("Autolock Plugin", false);
				autolockPlugin.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockPlugin, false);
				RegisterBool(autolockPlugin);
				
				autolockSkin = new JSONStorableBool("Autolock Skin", false);
				autolockSkin.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockSkin, false);
				RegisterBool(autolockSkin);
				
				autolockMorph = new JSONStorableBool("Autolock Morph", false);
				autolockMorph.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockMorph, false);
				RegisterBool(autolockMorph);
				
				autolockHair = new JSONStorableBool("Autolock Hair", false);
				autolockHair.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockHair, false);
				RegisterBool(autolockHair);
				
				autolockClothing = new JSONStorableBool("Autolock Clothing", false);
				autolockClothing.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(autolockClothing, false);
				RegisterBool(autolockClothing);
				
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>VAMLock</b> is a plugin that allows you to enable or disable the lock feature of the preset manager automatically or through scripting.\n\n" +
					"This plugin wouldn't have been possible without the original code snippet from <b>VRStudy</b> shared on VAM's Hub.\n\n" +
					"The concept behind this tiny plugin is to allow you to lock presets on a Person atom to freeze an appearance even when you use merge load.\n\n" +
					"If you are a bit creative, you could even use that to create a <b>character customizer</b> at the beginning of a story, allowing the player to choose hairs, morphs, skin or clothing presets.\n\n" +
					"The autolock checkboxes on the left will force the lock on the current atom at load.\n\n" +
					"If you want to lock or unlock elements of the preset manager during a playthrough, you will have to use triggers with the <b>Lock Parameter</b> and <b>Unlock Parameter</b> functions.\n\n" +
					"Please refer to <b>VAMLock Demo</b> to see a use case or the help section in the hub release of the plugin." +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				
				List<string> lockParamsNames = new List<string>();
				lockParamsNames.Add("Appearance");
				lockParamsNames.Add("Pose");
				lockParamsNames.Add("FemaleGlutePhysics");
				lockParamsNames.Add("FemaleBreastPhysics");
				lockParamsNames.Add("Plugin");
				lockParamsNames.Add("Skin");
				lockParamsNames.Add("Morph");
				lockParamsNames.Add("Hair");
				lockParamsNames.Add("Clothing");
				lockParamsNames.Add("All");
				
				JSONStorableStringChooser lockPersonParam = new JSONStorableStringChooser("Lock Parameter", lockParamsNames, "", "lockPersonParam", (string paramName)=>
				{
					try
					{
						if( paramName != "All" ) {
							PresetLock(containingAtom, paramName, true);
						} else {
				            foreach (string prm in lockParamsNames)
							{
								// Locking only if it is not "all"
								if ( prm != "All" ) {
									PresetLock(containingAtom, prm, true);
								}
							}
						}
					}
					catch(Exception e)
					{
						SuperController.LogError("VAMLock - Exception caught: " + e);
					}

				});
				RegisterStringChooser(lockPersonParam);
				
				JSONStorableStringChooser unlockPersonParam = new JSONStorableStringChooser("Unlock Parameter", lockParamsNames, "", "unlockPersonParam", (string paramName)=>
				{
					try
					{
						if( paramName != "All" ) {
							PresetLock(containingAtom, paramName, false);
						} else {
				            foreach (string prm in lockParamsNames)
							{
								// Unlocking only if it is not "all"
								if ( prm != "All" ) {
									PresetLock(containingAtom, prm, false);
								}
							}
						}
					}
					catch(Exception e)
					{
						SuperController.LogError("VAMLock - Exception caught: " + e);
					}

				});
				RegisterStringChooser(unlockPersonParam);
			}
            catch(Exception e)
            {
                SuperController.LogError("VAMLock - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			/// ----------------------------
			/// Auto locking at start
			/// ----------------------------	
			if( autolockAppearance.val == true ) {
				PresetLock(containingAtom, "Appearance", true);
			}
			
			if( autolockPose.val == true ) {
				PresetLock(containingAtom, "Pose", true);
			}
			
			if( autolockFemaleGlutePhysics.val == true ) {
				PresetLock(containingAtom, "FemaleGlutePhysics", true);
			}
			
			if( autolockFemaleBreastPhysics.val == true ) {
				PresetLock(containingAtom, "FemaleBreastPhysics", true);
			}
			
			if( autolockPlugin.val == true ) {
				PresetLock(containingAtom, "Plugin", true);
			}
			
			if( autolockSkin.val == true ) {
				PresetLock(containingAtom, "Skin", true);
			}

			if( autolockMorph.val == true ) {
				PresetLock(containingAtom, "Morph", true);
			}

			if( autolockHair.val == true ) {
				PresetLock(containingAtom, "Hair", true);
			}

			if( autolockClothing.val == true ) {
				PresetLock(containingAtom, "Clothing", true);
			}
        }
		
        /// <summary>
        /// Function to set the lock state of the presets.
        /// </summary>
        /// <param name="atom">A valid person atom</param>
        /// <param name="presetName"> Names returned from preset manager controls
        /// rescale2
        /// geometry
        /// AppearancePresets
        /// PosePresets
        /// FemaleGlutePhysicsPresets
        /// FemaleBreastPhysicsPresets
        /// PluginPresets
        /// SkinPresets
        /// MorphPresets
        /// HairPresets
        /// ClothingPresets
        /// <param name="locked">Whether locked or not. true/false</param>
        public void PresetLock(Atom atom, string presetName, bool locked = false)
        {
            // Make sure we have some valid data
            if (atom == null || atom.presetManagerControls == null || presetName == "") return;

            // Go through each PresetManagerControl
            foreach (MeshVR.PresetManagerControl ctrl in atom.presetManagerControls)
            {
                // If we find the name then set it
                if (ctrl.name.Contains(presetName))
                {
                    ctrl.lockParams = locked;
                }
            }
        }
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
	}
}